# Base class for operating-system-dependent actions (bootstrapping, etc)
#
# Copyright 2006-2007 Novell, Inc.
# Author: Charles Coffing <ccoffing@novell.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 

from exceptions import *

class VMActions:
    """Actions that are specific to a particular OS type within a VM."""

    def getInstallImagesFromPath(self, options, root, progress=lambda f:None, supports_32nonpae=True):
        """Locates or creates a Xen-enabled kernel and initrd under
        'root' to bootstrap install.
        """
        progress(1.0)
        return (None, None)

    def getInstallImagesFromUrl(self, options, url, progress=lambda f:None, supports_32nonpae=True):
        """Locates or creates a Xen-enabled kernel and initrd from
        'url' to bootstrap install.
        """
        raise InstSrcError(err=InstSrcError.E_UNSUPPORTED)

    def getRuntimeImagesFromPath(self, root, os_type):
        """Locates a Xen-enabled kernel and initrd under 'root' for
        runtime.
        """
        return (None, None)

    def getInstallArgs(self, options):
        """Return kernel arguments for installs."""
        return ''

    def getRuntimeArgs(self, options):
        """Return kernel arguments for runtime."""
        return ''

    def preprocessOptions(self, options):
        """Gives the guest a chance to massage options, after loading
        options from XML and command line, but before interacting with
        user.

        MUST BE IDEMPOTENT!
        """
        pass

    def setupOsSettings(self, guest, progress=lambda f:None):
        """Prepares the guest to use the guest.os_settings file or URL.

        For example, this may create a temporary disk image to contain
        guest.os_settings, and add the disk to guest.tmpdisks.
        """
        progress(1.0)

    def postInstall(self, guest):
        """Give the guest a chance to massage options, after first stage
        install shuts down, immediately before the config file is rewritten.
        """
        pass
