# CPUID class
#
# Copyright 2006-2008 Novell, Inc.
# Author: Charles Arnold <carnold@novell.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 

import os
import re
import string
import types

import caps
from exceptions import *
import msg


def get_platform_cpuid():
    flags = caps.cpu_flags()
    # FIXME: Parse flags for cpuid information
    return None

def is_hex(value):
    try:
        if isinstance(value, types.StringTypes):
            return int(value, 16)
        else:
            return value
    except ValueError:
        return None

def validate_reg(reg):
    # Valid syntax is either Hex or a string representing all bits
    # Examples: 0x06b1, reg=xxxxxxxxxx0000xx00xxx0000000xx0
    if reg.startswith('0x'):
        value = reg.split("0x",1)
        if is_hex(value[1]) is None:
            raise ParamError(err=ParamError.E_INVALID, details=reg)
    else:
        # Must be 32 characters long, can only consist of '1, 0, x, k, s'
        if len(reg) != 32:
            raise ParamError(err=ParamError.E_INVALID, details=reg)
        valid_chars = 'x,0,1,k,s'
        for c in reg:
            if c not in valid_chars:
                raise ParamError(err=ParamError.E_INVALID, details=reg)


def parse_string(cpuid_data):
    cpuids = []
    if cpuid_data == "":
        return cpuids.append(XenCPUID())

    functions = []
    function=None
    eax = None
    ebx = None
    ecx = None
    edx = None

    params = cpuid_data.split(',')
    for p in params:
        funcs = p.split(':')
        for f in funcs:
            if f.startswith('eax='):
                eax = f[4:]
                validate_reg(eax)
            elif f.startswith('ebx='):
                ebx = f[4:]
                validate_reg(ebx)
            elif f.startswith('ecx='):
                ecx = f[4:]
                validate_reg(ecx)
            elif f.startswith('edx='):
                edx = f[4:]
                validate_reg(edx)
            else:
                if f.startswith("0x"):
                    value = f.split("0x",1)[1]
                else:
                    value = f
                if is_hex(value) is not None:
                    if function:
                        if eax == None and ebx == None and ecx == None and edx == None:
                            function = function + ',' + f
                            continue
                        if function in functions:
                            raise ParamError(err=ParamError.E_INVALID, details=function)
                        cpuid = XenCPUID(function, eax, ebx, ecx, edx)
                        cpuids.append(cpuid)
                        functions.append(function)
                        eax = ebx = ecx = edx = None
                    function = f
                    continue
                else:
                    raise ParamError(err=ParamError.E_INVALID, details=p)
    if function:
        if eax == None and ebx == None and ecx == None and edx == None:
            raise ParamError(err=ParamError.E_INVALID, details=function)
        if function in functions:
            raise ParamError(err=ParamError.E_INVALID, details=function)
        cpuid = XenCPUID(function, eax, ebx, ecx, edx)
        cpuids.append(cpuid)
    return cpuids

class XenCPUID(object):

    def __init__(self, function=None, eax=None, ebx=None, ecx=None, edx=None):
        self.function = function
        self.eax = eax
        self.ebx = ebx
        self.ecx = ecx
        self.edx = edx
        
    def get_formatted(self):
        if self.function:
            padding = '              '
            flen = len(self.function)
            pad = padding[flen:]

            value = "\'%s:" % self.function
            if self.eax:
                value += "eax=%s" % self.eax
            if self.ebx:
                if self.eax:
                    if not self.ebx.startswith("0x"):
                        value += ",\n" + pad
                    else:
                        value += ","
                value += "ebx=%s" % self.ebx
            if self.ecx:
                if self.eax or self.ebx:
                    if not self.ecx.startswith("0x"):
                        value += ",\n" + pad
                    else:
                        value += ","
                value += "ecx=%s" % self.ecx
            if self.edx:
                if self.eax or self.ebx or self.ecx:
                    if not self.edx.startswith("0x"):
                        value += ",\n" + pad
                    else:
                        value += ","
                value += "edx=%s" % self.edx
            value += "\'"
            return value

    def get_function(self):
        return self.function

